/**
 * 计算三角形的内心坐标
 * @param p1 三角形的顶点 p1
 * @param p2 三角形的顶点 p2
 * @param p3 三角形的顶点 p3
 * @return 返回三角形的内心坐标
 */
function in_center_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const a1 = Math.hypot(p2.x - p3.x, p2.y - p3.y);
	const a2 = Math.hypot(p3.x - p1.x, p3.y - p1.y);
	const a3 = Math.hypot(p1.x - p2.x, p1.y - p2.y);
	const s = a1 + a2 + a3;
	const x = (a1 * p1.x + a2 * p2.x + a3 * p3.x) / s;
	const y = (a1 * p1.y + a2 * p2.y + a3 * p3.y) / s;
	return {'x': x, 'y': y};
}
/**
 * 计算三角形的旁心坐标
 * @param p1 三角形的顶点 p1
 * @param p2 三角形的顶点 p2
 * @param p3 三角形的顶点 p3
 * @return 返回三角形的顶点 p2 的旁心坐标
 */
function ex_center_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const a1 = Math.hypot(p2.x - p3.x, p2.y - p3.y);
	const a2 = Math.hypot(p3.x - p1.x, p3.y - p1.y);
	const a3 = Math.hypot(p1.x - p2.x, p1.y - p2.y);
	const s = a1 - a2 + a3;
	const x = (a1 * p1.x - a2 * p2.x + a3 * p3.x) / s;
	const y = (a1 * p1.y - a2 * p2.y + a3 * p3.y) / s;
	return {'x': x, 'y': y};
}
/**
 * 计算三角形的外心坐标
 * @param p1 三角形的顶点 p1
 * @param p2 三角形的顶点 p2
 * @param p3 三角形的顶点 p3
 * @return 返回三角形的外心坐标
 */
function circum_center_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const a1 = Math.hypot(p2.x - p3.x, p2.y - p3.y);
	const a2 = Math.hypot(p3.x - p1.x, p3.y - p1.y);
	const a3 = Math.hypot(p1.x - p2.x, p1.y - p2.y);
	const s = (a1 * a1 + a2 * a2 + a3 * a3) / 2;
	const [s1, s2, s3] = [s - a1 * a1, s - a2 * a2, s - a3 * a3];
	const t = (s1 * s2 * s3) / (s1 * s2 + s2 * s3 + s3 * s1);
	const [t1, t2, t3] = [t / s1, t / s2, t / s3];
	const x = ((1 - t1) * p1.x + (1 - t2) * p2.x + (1 - t3) * p3.x) / 2;
	const y = ((1 - t1) * p1.y + (1 - t2) * p2.y + (1 - t3) * p3.y) / 2;
	return {'x': x, 'y': y};
}
/**
 * 计算三角形的重心坐标
 * @param p1 三角形的顶点 p1
 * @param p2 三角形的顶点 p2
 * @param p3 三角形的顶点 p3
 * @return 返回三角形的重心坐标
 */
function median_center_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const x = (p1.x + p2.x + p3.x) / 3;
	const y = (p1.y + p2.y + p3.y) / 3;
	return {'x': x, 'y': y};
}
/**
 * 计算三角形的垂心坐标
 * @param p1 三角形的顶点 p1
 * @param p2 三角形的顶点 p2
 * @param p3 三角形的顶点 p3
 * @return 返回三角形的垂心坐标
 */
function ortho_center_xy(p1, p2, p3) {
	if (!p1.exist || !p2.exist || !p3.exist) return null;
	const a1 = Math.hypot(p2.x - p3.x, p2.y - p3.y);
	const a2 = Math.hypot(p3.x - p1.x, p3.y - p1.y);
	const a3 = Math.hypot(p1.x - p2.x, p1.y - p2.y);
	const s = (a1 * a1 + a2 * a2 + a3 * a3) / 2;
	const [s1, s2, s3] = [s - a1 * a1, s - a2 * a2, s - a3 * a3];
	const t = (s1 * s2 * s3) / (s1 * s2 + s2 * s3 + s3 * s1);
	const [t1, t2, t3] = [t / s1, t / s2, t / s3];
	const x = t1 * p1.x + t2 * p2.x + t3 * p3.x;
	const y = t1 * p1.y + t2 * p2.y + t3 * p3.y;
	return {'x': x, 'y': y};
}
